@@ -1,10 +1,8 @@
 package Test::Smoke;
 use strict;
 
-# $Id: Smoke.pm 1374 2012-06-17 11:36:11Z abeltje $
-use vars qw( $VERSION $REVISION $conf @EXPORT );
-$VERSION  = '1.53';
-$REVISION = __get_ts_patchlevel();
+use vars qw($VERSION $conf @EXPORT);
+$VERSION  = '1.59';
 
 use base 'Exporter';
 @EXPORT  = qw( $conf &read_config &run_smoke );
@@ -37,9 +35,9 @@ Test::Smoke - The Perl core test smoke suite
 
 C<Test::Smoke> exports C<$conf> and C<read_config()> by default.
 
-=over 4
+=head2 Test::Smoke::read_config( $config_name )
 
-=item Test::Smoke::read_config( $config_name )
+Read (require) the configfile.
 
 =cut
 
@@ -59,7 +57,7 @@ sub read_config {
     return defined $ConfigError ? undef : 1;
 }
 
-=item Test::Smoke->config_error()
+=head2 Test::Smoke->config_error()
 
 Return the value of C<$ConfigError>
 
@@ -69,7 +67,7 @@ sub config_error {
     return $ConfigError;
 }
 
-=item is_win32( )
+=head2 is_win32( )
 
 C<is_win32()> returns true if  C<< $^O eq "MSWin32" >>.
 
@@ -77,7 +75,7 @@ C<is_win32()> returns true if  C<< $^O eq "MSWin32" >>.
 
 sub is_win32() { $^O eq "MSWin32" }
 
-=item do_manifest_check( $ddir, $smoker )
+=head2 do_manifest_check( $ddir, $smoker )
 
 C<do_manifest_check()> uses B<Test::Smoke::SourceTree> to do the 
 MANIFEST check.
@@ -98,7 +96,7 @@ sub do_manifest_check {
     }
 }
 
-=item set_smoke_patchlevel( $ddir, $patch[, $verbose] )
+=head2 set_smoke_patchlevel( $ddir, $patch[, $verbose] )
 
 Set the current patchlevel as a registered patch like "SMOKE$patch"
 
@@ -114,7 +112,7 @@ sub set_smoke_patchlevel {
     @smokereg or set_local_patch( $ddir, "SMOKE$patch" );
 }
 
-=item run_smoke( [$continue[, @df_buildopts]] )
+=head2 run_smoke( [$continue[, @df_buildopts]] )
 
 C<run_smoke()> sets up de build environment and gets the private Policy
 file and build configurations and then runs the smoke stuff for all 
@@ -176,7 +174,8 @@ sub run_smoke {
     chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
     unless ( $continue ) {
         $smoker->make_distclean( );
-        $smoker->ttylog( "Smoking patch $patch->[0] $patch->[-1]\n" ); 
+        $smoker->ttylog("Smoking patch $patch->[0] $patch->[1]\n"); 
+        $smoker->ttylog("Smoking branch $patch->[2]\n") if $patch->[2];
         do_manifest_check( $conf->{ddir}, $smoker );
         set_smoke_patchlevel( $conf->{ddir}, $patch->[0] );
     }
@@ -193,7 +192,7 @@ sub run_smoke {
         $smoker->smoke( $this_cfg, $Policy );
     }
 
-    $smoker->ttylog( "Finished smoking $patch->[0] $patch->[-1]\n" );
+    $smoker->ttylog( "Finished smoking $patch->[0] $patch->[1] $patch->[2]\n" );
     $smoker->mark_out;
 
     close LOG or do {
@@ -202,37 +201,8 @@ sub run_smoke {
    };
 }
 
-=item __get_ts_patchlevel( )
-
-Read the contents of F<.patch>.
-
-=cut
-
-use FindBin;
-use File::Spec::Functions;
-
-sub __get_ts_patchlevel {
-    my( $rev ) = q$Rev: 1374 $ =~ /(\d+)/;
-    if ( ! $rev && -d '.git' ) {
-       chomp( $rev = `git describe --all --long` );
-    }
-    my $dotpatch = catfile $FindBin::Bin, '.patch';
-    local *DOTPATCH;
-    open DOTPATCH, "< $dotpatch" or return $rev;
-    chomp( my $plevel = <DOTPATCH> );
-    close DOTPATCH;
-    ! defined $plevel or $plevel = $rev;
-    return $plevel > $rev ? $plevel : $rev;
-}
-
 1;
 
-=back
-
-=head1 REVISION
-
-$Id: Smoke.pm 1374 2012-06-17 11:36:11Z abeltje $
-
 =head1 COPYRIGHT
 
 (c) 2003, All rights reserved.
